@@ -1,14 +1,12 @@
 /*
  * This file is part of GPaste.
  *
- * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Gettext = imports.gettext;
 
-const Gtk = imports.gi.Gtk;
-
 const GPaste = imports.gi.GPaste;
 
 const ExtensionUtils = imports.misc.extensionUtils;
@@ -21,7 +19,7 @@ function init() {
 }
 
 function buildPrefsWidget() {
-    let widget = new GPaste.SettingsUiWidget({ orientation: Gtk.Orientation.VERTICAL, margin: 12 });
+    let widget = new GPaste.SettingsUiWidget({ margin: 12 });
     if (widget) {
         widget.show_all();
     }
