@@ -1,7 +1,7 @@
 /*
  * This file is part of GPaste.
  *
- * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
@@ -11,7 +11,7 @@ const GPaste = imports.gi.GPaste;
 
 const St = imports.gi.St;
 
-const GPasteUiItem = new Lang.Class({
+var GPasteUiItem = new Lang.Class({
     Name: 'GPasteUiItem',
 
     _init: function(menu) {
