@@ -1,7 +1,7 @@
 /*
  * This file is part of GPaste.
  *
- * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-client.h>
@@ -192,7 +192,8 @@ on_elements_ready (GObject      *source_object G_GNUC_UNUSED,
 
         append_dict_entry (&dict, "id", index);
         append_dict_entry (&dict, "name", result);
-        append_dict_entry (&dict, "gicon", "gtk-edit-paste");
+        append_dict_entry (&dict, "gicon", G_PASTE_ICON_NAME);
+        append_dict_entry (&dict, "clipboardText", results[i]);
 
         g_variant_builder_add_value (&builder, g_variant_builder_end (&dict));
     }
@@ -302,7 +303,7 @@ g_paste_search_provider_unregister_object (gpointer user_data)
 static void
 g_paste_search_provider_dispose (GObject *object)
 {
-    const GPasteSearchProviderPrivate *priv = _g_paste_search_provider_get_instance_private (G_PASTE_SEARCH_PROVIDER (object));
+    GPasteSearchProviderPrivate *priv = g_paste_search_provider_get_instance_private (G_PASTE_SEARCH_PROVIDER (object));
 
     if (priv->connection)
     {
