@@ -1,20 +1,7 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-keybinder.h>
@@ -34,6 +21,13 @@ struct _GPasteKeybinder
     GObject parent_instance;
 };
 
+enum
+{
+    C_ACCEL,
+
+    C_LAST_SIGNAL
+};
+
 typedef struct
 {
     GSList                 *keybindings;
@@ -44,11 +38,12 @@ typedef struct
     gboolean                grabbing;
     guint64                 retries;
 
-    gulong                  accel_signal;
     guint64                 shell_watch;
+
+    guint64                 c_signals[C_LAST_SIGNAL];
 } GPasteKeybinderPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteKeybinder, g_paste_keybinder, G_TYPE_OBJECT)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (Keybinder, keybinder, G_TYPE_OBJECT)
 
 /***************************/
 /* Internal grabbing stuff */
@@ -65,7 +60,7 @@ g_paste_keybinder_change_grab_wayland (void)
 #if defined(ENABLE_X_KEYBINDER) && defined (GDK_WINDOWING_X11)
 static void
 g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
-                                   Display          *display,
+                                   GdkDisplay       *display,
                                    gboolean          grab)
 {
     if (!g_paste_keybinding_is_active (binding))
@@ -73,10 +68,11 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
 
     guchar mask_bits[XIMaskLen (XI_LASTEVENT)] = { 0 };
     XIEventMask mask = { XIAllMasterDevices, sizeof (mask_bits), mask_bits };
+    Display *xdisplay = GDK_DISPLAY_XDISPLAY (display);
 
     XISetMask (mask.mask, XI_KeyPress);
 
-    gdk_error_trap_push ();
+    gdk_x11_display_error_trap_push (display);
 
     guint64 mod_masks [] = {
         0, /* modifier only */
@@ -99,7 +95,7 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
         {
             if (grab)
             {
-                XIGrabKeycode (display,
+                XIGrabKeycode (xdisplay,
                                XIAllMasterDevices,
                                *keycode,
                                window,
@@ -112,7 +108,7 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
             }
             else
             {
-                XIUngrabKeycode (display,
+                XIUngrabKeycode (xdisplay,
                                  XIAllMasterDevices,
                                  *keycode,
                                  window,
@@ -122,8 +118,8 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
         }
     }
 
-    gdk_flush ();
-    gdk_error_trap_pop_ignored ();
+    gdk_display_flush (display);
+    gdk_x11_display_error_trap_pop_ignored (display);
 }
 #endif
 
@@ -140,7 +136,7 @@ g_paste_keybinder_change_grab_internal (GPasteKeybinding *binding,
 #endif
 #if defined(ENABLE_X_KEYBINDER) && defined (GDK_WINDOWING_X11)
     if (GDK_IS_X11_DISPLAY (display))
-        g_paste_keybinder_change_grab_x11 (binding, GDK_DISPLAY_XDISPLAY (display), grab);
+        g_paste_keybinder_change_grab_x11 (binding, display, grab);
     else
 #endif
         g_warning ("Unsupported GDK backend, keybinder won't work.");
@@ -150,6 +146,13 @@ g_paste_keybinder_change_grab_internal (GPasteKeybinding *binding,
 /* Wrapper around GPasteKeybinding */
 /***********************************/
 
+enum
+{
+    C_K_REBIND,
+
+    C_K_LAST_SIGNAL
+};
+
 typedef struct
 {
     GPasteKeybinding       *binding;
@@ -159,7 +162,7 @@ typedef struct
 
     guint32                 action;
 
-    gulong                  rebind_signal;
+    guint64                 c_signals[C_K_LAST_SIGNAL];
 } _Keybinding;
 
 static void
@@ -270,26 +273,30 @@ _keybinding_new (GPasteKeybinding       *binding,
 
     k->action = 0;
 
-    g_autofree gchar *detailed_signal = g_strdup_printf ("rebind::%s",
-                                                                   g_paste_keybinding_get_dconf_key (binding));
-    k->rebind_signal = g_signal_connect_swapped (settings,
-                                                 detailed_signal,
-                                                 G_CALLBACK (_keybinding_rebind),
-                                                 k);
+    g_autofree gchar *detailed_signal = g_strdup_printf ("rebind::%s", g_paste_keybinding_get_dconf_key (binding));
+
+    k->c_signals[C_K_REBIND] = g_signal_connect_swapped (settings,
+                                                         detailed_signal,
+                                                         G_CALLBACK (_keybinding_rebind),
+                                                         k);
     return k;
 }
 
 static void
-_keybinding_cleanup_shell_stuff (_Keybinding *k)
+_keybinding_cleanup_shell_stuff (gpointer data,
+                                 gpointer user_data G_GNUC_UNUSED)
 {
+    _Keybinding *k = data;
     k->action = 0;
 }
 
 static void
-_keybinding_free (_Keybinding *k)
+_keybinding_free (gpointer data,
+                  gpointer user_data G_GNUC_UNUSED)
 {
+    _Keybinding *k = data;
     _keybinding_ungrab (k);
-    g_signal_handler_disconnect (k->settings, k->rebind_signal);
+    g_signal_handler_disconnect (k->settings, k->c_signals[C_K_REBIND]);
     g_object_unref (k->binding);
     g_object_unref (k->settings);
     g_clear_object (&k->shell_client);
@@ -304,15 +311,13 @@ _keybinding_free (_Keybinding *k)
  * @binding: (transfer full): a #GPasteKeybinding instance
  *
  * Add a new keybinding
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_keybinder_add_keybinding (GPasteKeybinder  *self,
                                   GPasteKeybinding *binding)
 {
-    g_return_if_fail (G_PASTE_IS_KEYBINDER (self));
-    g_return_if_fail (G_PASTE_IS_KEYBINDING (binding));
+    g_return_if_fail (_G_PASTE_IS_KEYBINDER (self));
+    g_return_if_fail (_G_PASTE_IS_KEYBINDING (binding));
 
     GPasteKeybinderPrivate *priv = g_paste_keybinder_get_instance_private (self);
 
@@ -430,13 +435,11 @@ g_paste_keybinder_private_grab_all_gnome_shell (GPasteKeybinderPrivate *priv)
  * @self: a #GPasteKeybinder instance
  *
  * Activate all the managed keybindings
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_keybinder_activate_all (GPasteKeybinder *self)
 {
-    g_return_if_fail (G_PASTE_IS_KEYBINDER (self));
+    g_return_if_fail (_G_PASTE_IS_KEYBINDER (self));
 
     GPasteKeybinderPrivate *priv = g_paste_keybinder_get_instance_private (self);
 
@@ -463,15 +466,13 @@ g_paste_keybinder_deactivate_keybinding_func (gpointer data,
  * @self: a #GPasteKeybinder instance
  *
  * Deactivate all the managed keybindings
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_keybinder_deactivate_all (GPasteKeybinder *self)
 {
-    g_return_if_fail (G_PASTE_IS_KEYBINDER (self));
+    g_return_if_fail (_G_PASTE_IS_KEYBINDER (self));
 
-    GPasteKeybinderPrivate *priv = g_paste_keybinder_get_instance_private (self);
+    const GPasteKeybinderPrivate *priv = _g_paste_keybinder_get_instance_private (self);
 
     g_slist_foreach (priv->keybindings,
                      g_paste_keybinder_deactivate_keybinding_func,
@@ -538,19 +539,18 @@ g_paste_keybinder_filter (GdkXEvent *xevent,
                           gpointer   data)
 {
     GPasteKeybinderPrivate *priv = data;
+    GdkDisplay *display = gdk_display_get_default ();
 
-    for (GList *dev = gdk_device_manager_list_devices (gdk_display_get_device_manager (gdk_display_get_default ()),
-                                                       GDK_DEVICE_TYPE_MASTER); dev; dev = g_list_next (dev))
+    for (GList *_seat = gdk_display_list_seats (display); _seat; _seat = g_list_next (_seat))
     {
-        GdkDevice *device = dev->data;
+        GdkSeat *seat = _seat->data;
 
-        if (gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD)
-            gdk_device_ungrab (device, GDK_CURRENT_TIME);
+        if (gdk_seat_get_keyboard (seat))
+            gdk_seat_ungrab (seat);
     }
 
-    gdk_flush ();
+    gdk_display_flush (display);
 
-    GdkDisplay *display = gdk_display_get_default ();
     GdkModifierType modifiers = 0;
     guint64 keycode = 0;
 
@@ -609,7 +609,7 @@ on_shell_appeared (GDBusConnection *connection G_GNUC_UNUSED,
 static void
 g_paste_keybinder_private_cleanup_shell_stuff (GPasteKeybinderPrivate *priv)
 {
-    g_slist_foreach (priv->keybindings, (GFunc) _keybinding_cleanup_shell_stuff, NULL);
+    g_slist_foreach (priv->keybindings, _keybinding_cleanup_shell_stuff, NULL);
 }
 
 static void
@@ -637,7 +637,7 @@ g_paste_keybinder_dispose (GObject *object)
 
     if (priv->shell_client)
     {
-        g_signal_handler_disconnect (priv->shell_client, priv->accel_signal);
+        g_signal_handler_disconnect (priv->shell_client, priv->c_signals[C_ACCEL]);
         g_clear_object (&priv->shell_client);
     }
 
@@ -645,7 +645,7 @@ g_paste_keybinder_dispose (GObject *object)
     {
         g_clear_object (&priv->settings);
         g_paste_keybinder_deactivate_all (self);
-        g_slist_foreach (priv->keybindings, (GFunc) _keybinding_free, NULL);
+        g_slist_foreach (priv->keybindings, _keybinding_free, NULL);
         priv->keybindings = NULL;
     }
 
@@ -707,8 +707,8 @@ G_PASTE_VISIBLE GPasteKeybinder *
 g_paste_keybinder_new (GPasteSettings         *settings,
                        GPasteGnomeShellClient *shell_client)
 {
-    g_return_val_if_fail (G_PASTE_IS_SETTINGS (settings), NULL);
-    g_return_val_if_fail (!shell_client || G_PASTE_IS_GNOME_SHELL_CLIENT (shell_client), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), NULL);
+    g_return_val_if_fail (!shell_client || _G_PASTE_IS_GNOME_SHELL_CLIENT (shell_client), NULL);
 
     GPasteKeybinder *self = G_PASTE_KEYBINDER (g_object_new (G_PASTE_TYPE_KEYBINDER, NULL));
     GPasteKeybinderPrivate *priv = g_paste_keybinder_get_instance_private (self);
@@ -721,10 +721,10 @@ g_paste_keybinder_new (GPasteSettings         *settings,
 
     if (shell_client)
     {
-        priv->accel_signal = g_signal_connect (shell_client,
-                                               "accelerator-activated",
-                                               G_CALLBACK (on_accelerator_activated),
-                                               priv);
+        priv->c_signals[C_ACCEL] = g_signal_connect (shell_client,
+                                                     "accelerator-activated",
+                                                     G_CALLBACK (on_accelerator_activated),
+                                                     priv);
         priv->shell_watch = g_bus_watch_name (G_BUS_TYPE_SESSION,
                                               G_PASTE_GNOME_SHELL_BUS_NAME,
                                               G_BUS_NAME_WATCHER_FLAGS_NONE,
