@@ -1,32 +1,26 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2013-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-client.h>
 #include <gpaste-gsettings-keys.h>
-#include <gpaste-settings.h>
 #include <gpaste-settings-ui-stack.h>
+#include <gpaste-util.h>
 
 struct _GPasteSettingsUiStack
 {
     GtkStack parent_instance;
 };
 
+enum
+{
+    C_SETTINGS,
+
+    C_LAST_SIGNAL
+};
+
 typedef struct
 {
     GPasteClient    *client;
@@ -34,6 +28,7 @@ typedef struct
 
     GError          *init_error;
 
+    GtkSwitch       *close_on_select_switch;
     GtkSwitch       *images_support_switch;
     GtkSwitch       *growing_lines_switch;
     GtkSwitch       *primary_to_history_switch;
@@ -59,13 +54,13 @@ typedef struct
     GtkSwitch       *extension_enabled_switch;
     GtkSwitch       *track_extension_state_switch;
 
-    gulong           settings_signal;
+    guint64          c_signals[C_LAST_SIGNAL];
 } GPasteSettingsUiStackPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteSettingsUiStack, g_paste_settings_ui_stack, GTK_TYPE_STACK)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (SettingsUiStack, settings_ui_stack, GTK_TYPE_STACK)
 
 #define SETTING_CALLBACK_FULL(setting, type, cast)                            \
-    static void                                                               \
+    static inline void                                                        \
     setting##_callback (type value,                                           \
                         gpointer user_data)                                   \
     {                                                                         \
@@ -87,8 +82,6 @@ G_DEFINE_TYPE_WITH_PRIVATE (GPasteSettingsUiStack, g_paste_settings_ui_stack, GT
  * @panel: (transfer none): the #GPasteSettingsUiPanel to add
  *
  * Add a new panel to the #GPasteSettingsUiStack
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_settings_ui_stack_add_panel (GPasteSettingsUiStack *self,
@@ -96,25 +89,23 @@ g_paste_settings_ui_stack_add_panel (GPasteSettingsUiStack *self,
                                      const gchar           *label,
                                      GPasteSettingsUiPanel *panel)
 {
-    g_return_if_fail (G_PASTE_IS_SETTINGS_UI_STACK (self));
+    g_return_if_fail (_G_PASTE_IS_SETTINGS_UI_STACK (self));
 
     gtk_stack_add_titled (GTK_STACK (self),
                           GTK_WIDGET (panel),
                           name, label);
 }
 
+BOOLEAN_CALLBACK (close_on_select)
+BOOLEAN_CALLBACK (extension_enabled)
 BOOLEAN_CALLBACK (growing_lines)
 BOOLEAN_CALLBACK (images_support)
 BOOLEAN_CALLBACK (primary_to_history)
 BOOLEAN_CALLBACK (save_history)
 BOOLEAN_CALLBACK (synchronize_clipboards)
 BOOLEAN_CALLBACK (track_changes)
-BOOLEAN_CALLBACK (trim_items)
-
-#ifdef ENABLE_EXTENSION
-BOOLEAN_CALLBACK (extension_enabled)
 BOOLEAN_CALLBACK (track_extension_state)
-#endif
+BOOLEAN_CALLBACK (trim_items)
 
 static GPasteSettingsUiPanel *
 g_paste_settings_ui_stack_private_make_behaviour_panel (GPasteSettingsUiStackPrivate *priv)
@@ -128,20 +119,29 @@ g_paste_settings_ui_stack_private_make_behaviour_panel (GPasteSettingsUiStackPri
                                                                                 track_changes_callback,
                                                                                 (GPasteResetCallback) g_paste_settings_reset_track_changes,
                                                                                 settings);
-#ifdef ENABLE_EXTENSION
-    priv->extension_enabled_switch = g_paste_settings_ui_panel_add_boolean_setting (panel,
-                                                                                    _("Enable the gnome-shell extension"),
-                                                                                    g_paste_settings_get_extension_enabled (settings),
-                                                                                    extension_enabled_callback,
-                                                                                    NULL,
-                                                                                    settings);
-    priv->track_extension_state_switch = g_paste_settings_ui_panel_add_boolean_setting (panel,
-                                                                                        _("Sync the daemon state with the extension's one"),
-                                                                                        g_paste_settings_get_track_extension_state (settings),
-                                                                                        track_extension_state_callback,
-                                                                                        (GPasteResetCallback) g_paste_settings_reset_track_extension_state,
+    priv->close_on_select_switch = g_paste_settings_ui_panel_add_boolean_setting (panel,
+                                                                                  _("Close UI on select"),
+                                                                                  g_paste_settings_get_close_on_select (settings),
+                                                                                  close_on_select_callback,
+                                                                                  (GPasteResetCallback) g_paste_settings_reset_close_on_select,
+                                                                                  settings);
+
+    if (g_paste_util_has_gnome_shell ())
+    {
+        priv->extension_enabled_switch = g_paste_settings_ui_panel_add_boolean_setting (panel,
+                                                                                        _("Enable the gnome-shell extension"),
+                                                                                        g_paste_settings_get_extension_enabled (settings),
+                                                                                        extension_enabled_callback,
+                                                                                        NULL,
                                                                                         settings);
-#endif
+        priv->track_extension_state_switch = g_paste_settings_ui_panel_add_boolean_setting (panel,
+                                                                                            _("Sync the daemon state with the extension's one"),
+                                                                                            g_paste_settings_get_track_extension_state (settings),
+                                                                                            track_extension_state_callback,
+                                                                                            (GPasteResetCallback) g_paste_settings_reset_track_extension_state,
+                                                                                            settings);
+    }
+
     g_paste_settings_ui_panel_add_separator (panel);
     priv->primary_to_history_switch = g_paste_settings_ui_panel_add_boolean_setting (panel,
                                                                                      _("Primary selection affects history"),
@@ -199,42 +199,42 @@ g_paste_settings_ui_stack_private_make_history_settings_panel (GPasteSettingsUiS
     GPasteSettingsUiPanel *panel = g_paste_settings_ui_panel_new ();
 
     priv->element_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
-                                                                             _("Max element size when displaying: "),
+                                                                             _("Max element size when displaying"),
                                                                              (gdouble) g_paste_settings_get_element_size (settings),
                                                                              0, 511, 5,
                                                                              element_size_callback,
                                                                              (GPasteResetCallback) g_paste_settings_reset_element_size,
                                                                              settings);
     priv->max_displayed_history_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
-                                                                                           _("Max displayed history size: "),
+                                                                                           _("Max displayed history size"),
                                                                                            (gdouble) g_paste_settings_get_max_displayed_history_size (settings),
                                                                                            10, 255, 5,
                                                                                            max_displayed_history_size_callback,
                                                                                            (GPasteResetCallback) g_paste_settings_reset_max_displayed_history_size,
                                                                                            settings);
     priv->max_history_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
-                                                                                 _("Max history size: "),
+                                                                                 _("Max history size"),
                                                                                  (gdouble) g_paste_settings_get_max_history_size (settings),
                                                                                  100, 65535, 5,
                                                                                  max_history_size_callback,
                                                                                  (GPasteResetCallback) g_paste_settings_reset_max_history_size,
                                                                                  settings);
     priv->max_memory_usage_button = g_paste_settings_ui_panel_add_range_setting (panel,
-                                                                                 _("Max memory usage (MB): "),
+                                                                                 _("Max memory usage (MB)"),
                                                                                  (gdouble) g_paste_settings_get_max_memory_usage (settings),
                                                                                  5, 16383, 5,
                                                                                  max_memory_usage_callback,
                                                                                  (GPasteResetCallback) g_paste_settings_reset_max_memory_usage,
                                                                                  settings);
     priv->max_text_item_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
-                                                                                   _("Max text item length: "),
+                                                                                   _("Max text item length"),
                                                                                    (gdouble) g_paste_settings_get_max_text_item_size (settings),
                                                                                    1, G_MAXUINT64, 1,
                                                                                    max_text_item_size_callback,
                                                                                    (GPasteResetCallback) g_paste_settings_reset_max_text_item_size,
                                                                                    settings);
     priv->min_text_item_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
-                                                                                   _("Min text item length: "),
+                                                                                   _("Min text item length"),
                                                                                    (gdouble) g_paste_settings_get_min_text_item_size (settings),
                                                                                    1, 65535, 1,
                                                                                    min_text_item_size_callback,
@@ -260,49 +260,49 @@ g_paste_settings_ui_stack_private_make_keybindings_panel (GPasteSettingsUiStackP
 
     /* translators: Keyboard shortcut to delete the active item from history */
     priv->pop_entry = g_paste_settings_ui_panel_add_text_setting (panel,
-                                                                  _("Delete the active item from history: "),
+                                                                  _("Delete the active item from history"),
                                                                   g_paste_settings_get_pop (settings),
                                                                   pop_callback,
                                                                   (GPasteResetCallback) g_paste_settings_reset_pop,
                                                                   settings);
     /* translators: Keyboard shortcut to launch the graphical tool */
     priv->launch_ui_entry = g_paste_settings_ui_panel_add_text_setting (panel,
-                                                                        _("Launch the graphical tool: "),
+                                                                        _("Launch the graphical tool"),
                                                                         g_paste_settings_get_launch_ui (settings),
                                                                         launch_ui_callback,
                                                                         (GPasteResetCallback) g_paste_settings_reset_launch_ui,
                                                                         settings);
     /* translators: Keyboard shortcut to mark the active item as being a password */
     priv->make_password_entry = g_paste_settings_ui_panel_add_text_setting (panel,
-                                                                            _("Mark the active item as being a password: "),
+                                                                            _("Mark the active item as being a password"),
                                                                             g_paste_settings_get_make_password (settings),
                                                                             make_password_callback,
                                                                             (GPasteResetCallback) g_paste_settings_reset_make_password,
                                                                             settings);
     /* translators: Keyboard shortcut to display the history */
     priv->show_history_entry = g_paste_settings_ui_panel_add_text_setting (panel,
-                                                                           _("Display the history: "),
+                                                                           _("Display the history"),
                                                                            g_paste_settings_get_show_history (settings),
                                                                            show_history_callback,
                                                                            (GPasteResetCallback) g_paste_settings_reset_show_history,
                                                                            settings);
     /* translators: Keyboard shortcut to sync the clipboard to the primary selection */
     priv->sync_clipboard_to_primary_entry = g_paste_settings_ui_panel_add_text_setting (panel,
-                                                                                        _("Sync the clipboard to the primary selection: "),
+                                                                                        _("Sync the clipboard to the primary selection"),
                                                                                         g_paste_settings_get_sync_clipboard_to_primary (settings),
                                                                                         sync_clipboard_to_primary_callback,
                                                                                         (GPasteResetCallback) g_paste_settings_reset_sync_clipboard_to_primary,
                                                                                         settings);
     /* translators: Keyboard shortcut to sync the primary selection to the clipboard */
     priv->sync_primary_to_clipboard_entry = g_paste_settings_ui_panel_add_text_setting (panel,
-                                                                                        _("Sync the primary selection to the clipboard: "),
+                                                                                        _("Sync the primary selection to the clipboard"),
                                                                                         g_paste_settings_get_sync_primary_to_clipboard (settings),
                                                                                         sync_primary_to_clipboard_callback,
                                                                                         (GPasteResetCallback) g_paste_settings_reset_sync_primary_to_clipboard,
                                                                                         settings);
     /* translators: Keyboard shortcut to upload the active item from history to a pastebin service */
     priv->upload_entry = g_paste_settings_ui_panel_add_text_setting (panel,
-                                                                     _("Upload the active item to a pastebin service: "),
+                                                                     _("Upload the active item to a pastebin service"),
                                                                      g_paste_settings_get_upload (settings),
                                                                      upload_callback,
                                                                      (GPasteResetCallback) g_paste_settings_reset_upload,
@@ -326,8 +326,6 @@ g_paste_settings_ui_check_connection_error (GError *error)
  * @self: a #GPasteSettingsUiStack instance
  *
  * Fill the #GPasteSettingsUiStack with default panels
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_settings_ui_stack_fill (GPasteSettingsUiStack *self)
@@ -346,52 +344,55 @@ g_paste_settings_ui_stack_settings_changed (GPasteSettings *settings,
 {
     GPasteSettingsUiStackPrivate *priv = user_data;
 
-    if (!g_strcmp0 (key, G_PASTE_ELEMENT_SIZE_SETTING))
+    if (g_paste_str_equal (key, G_PASTE_CLOSE_ON_SELECT_SETTING))
+        gtk_switch_set_active (GTK_SWITCH (priv->close_on_select_switch), g_paste_settings_get_close_on_select (settings));
+    else if (g_paste_str_equal (key, G_PASTE_ELEMENT_SIZE_SETTING))
         gtk_spin_button_set_value (priv->element_size_button, g_paste_settings_get_element_size (settings));
-    else if (!g_strcmp0 (key, G_PASTE_GROWING_LINES_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_GROWING_LINES_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->growing_lines_switch), g_paste_settings_get_growing_lines (settings));
-    else if (!g_strcmp0 (key, G_PASTE_IMAGES_SUPPORT_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_IMAGES_SUPPORT_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->images_support_switch), g_paste_settings_get_images_support (settings));
-    else if (!g_strcmp0 (key, G_PASTE_LAUNCH_UI_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_LAUNCH_UI_SETTING))
         gtk_entry_set_text (priv->launch_ui_entry, g_paste_settings_get_launch_ui (settings));
-    else if (!g_strcmp0 (key, G_PASTE_MAKE_PASSWORD_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAKE_PASSWORD_SETTING))
         gtk_entry_set_text (priv->make_password_entry, g_paste_settings_get_make_password (settings));
-    else if (!g_strcmp0 (key, G_PASTE_MAX_DISPLAYED_HISTORY_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_DISPLAYED_HISTORY_SIZE_SETTING))
         gtk_spin_button_set_value (priv->max_displayed_history_size_button, g_paste_settings_get_max_displayed_history_size (settings));
-    else if (!g_strcmp0 (key, G_PASTE_MAX_HISTORY_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_HISTORY_SIZE_SETTING))
         gtk_spin_button_set_value (priv->max_history_size_button, g_paste_settings_get_max_history_size (settings));
-    else if (!g_strcmp0 (key, G_PASTE_MAX_MEMORY_USAGE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_MEMORY_USAGE_SETTING))
         gtk_spin_button_set_value (priv->max_memory_usage_button, g_paste_settings_get_max_memory_usage (settings));
-    else if (!g_strcmp0 (key, G_PASTE_MAX_TEXT_ITEM_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_TEXT_ITEM_SIZE_SETTING))
         gtk_spin_button_set_value (priv->max_text_item_size_button, g_paste_settings_get_max_text_item_size (settings));
-    else if (!g_strcmp0 (key, G_PASTE_MIN_TEXT_ITEM_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MIN_TEXT_ITEM_SIZE_SETTING))
         gtk_spin_button_set_value (priv->min_text_item_size_button, g_paste_settings_get_min_text_item_size (settings));
-    else if (!g_strcmp0 (key, G_PASTE_POP_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_POP_SETTING))
         gtk_entry_set_text (priv->pop_entry, g_paste_settings_get_pop (settings));
-    else if (!g_strcmp0 (key, G_PASTE_PRIMARY_TO_HISTORY_SETTING ))
+    else if (g_paste_str_equal (key, G_PASTE_PRIMARY_TO_HISTORY_SETTING ))
         gtk_switch_set_active (GTK_SWITCH (priv->primary_to_history_switch), g_paste_settings_get_primary_to_history (settings));
-    else if (!g_strcmp0 (key, G_PASTE_SAVE_HISTORY_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SAVE_HISTORY_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->save_history_switch), g_paste_settings_get_save_history (settings));
-    else if (!g_strcmp0 (key, G_PASTE_SHOW_HISTORY_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SHOW_HISTORY_SETTING))
         gtk_entry_set_text (priv->show_history_entry, g_paste_settings_get_show_history (settings));
-    else if (!g_strcmp0 (key, G_PASTE_SYNC_CLIPBOARD_TO_PRIMARY_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SYNC_CLIPBOARD_TO_PRIMARY_SETTING))
         gtk_entry_set_text (priv->sync_clipboard_to_primary_entry, g_paste_settings_get_sync_clipboard_to_primary (settings));
-    else if (!g_strcmp0 (key, G_PASTE_SYNC_PRIMARY_TO_CLIPBOARD_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SYNC_PRIMARY_TO_CLIPBOARD_SETTING))
         gtk_entry_set_text (priv->sync_primary_to_clipboard_entry, g_paste_settings_get_sync_primary_to_clipboard (settings));
-    else if (!g_strcmp0 (key, G_PASTE_UPLOAD_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_UPLOAD_SETTING))
         gtk_entry_set_text (priv->upload_entry, g_paste_settings_get_upload (settings));
-    else if (!g_strcmp0 (key, G_PASTE_SYNCHRONIZE_CLIPBOARDS_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SYNCHRONIZE_CLIPBOARDS_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->synchronize_clipboards_switch), g_paste_settings_get_synchronize_clipboards (settings));
-    else if (!g_strcmp0 (key, G_PASTE_TRACK_CHANGES_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_TRACK_CHANGES_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->track_changes_switch), g_paste_settings_get_track_changes (settings));
-    else if (!g_strcmp0 (key, G_PASTE_TRIM_ITEMS_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_TRIM_ITEMS_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->trim_items_switch), g_paste_settings_get_trim_items (settings));
-#ifdef ENABLE_EXTENSION
-    else if (!g_strcmp0 (key, G_PASTE_EXTENSION_ENABLED_SETTING))
-        gtk_switch_set_active (GTK_SWITCH (priv->extension_enabled_switch), g_paste_settings_get_extension_enabled (settings));
-    else if (!g_strcmp0 (key, G_PASTE_TRACK_EXTENSION_STATE_SETTING))
-        gtk_switch_set_active (GTK_SWITCH (priv->track_extension_state_switch), g_paste_settings_get_track_extension_state (settings));
-#endif
+    else if (g_paste_util_has_gnome_shell ())
+    {
+        if (g_paste_str_equal (key, G_PASTE_EXTENSION_ENABLED_SETTING))
+            gtk_switch_set_active (GTK_SWITCH (priv->extension_enabled_switch), g_paste_settings_get_extension_enabled (settings));
+        else if (g_paste_str_equal (key, G_PASTE_TRACK_EXTENSION_STATE_SETTING))
+            gtk_switch_set_active (GTK_SWITCH (priv->track_extension_state_switch), g_paste_settings_get_track_extension_state (settings));
+    }
 }
 
 static void
@@ -401,7 +402,7 @@ g_paste_settings_ui_stack_dispose (GObject *object)
 
     if (priv->settings) /* first dispose call */
     {
-        g_signal_handler_disconnect (priv->settings, priv->settings_signal);
+        g_signal_handler_disconnect (priv->settings, priv->c_signals[C_SETTINGS]);
         g_clear_object (&priv->settings);
         g_clear_object (&priv->client);
     }
@@ -412,7 +413,7 @@ g_paste_settings_ui_stack_dispose (GObject *object)
 static void
 g_paste_settings_ui_stack_finalize (GObject *object)
 {
-    GPasteSettingsUiStackPrivate *priv = g_paste_settings_ui_stack_get_instance_private (G_PASTE_SETTINGS_UI_STACK (object));
+    const GPasteSettingsUiStackPrivate *priv = _g_paste_settings_ui_stack_get_instance_private (G_PASTE_SETTINGS_UI_STACK (object));
     GStrv *actions = priv->actions;
 
     for (guint64 i = 0; actions[i]; ++i)
@@ -440,10 +441,10 @@ g_paste_settings_ui_stack_init (GPasteSettingsUiStack *self)
     priv->client = g_paste_client_new_sync (&priv->init_error);
 
     priv->settings = g_paste_settings_new ();
-    priv->settings_signal = g_signal_connect (priv->settings,
-                                              "changed",
-                                              G_CALLBACK (g_paste_settings_ui_stack_settings_changed),
-                                              priv);
+    priv->c_signals[C_SETTINGS] = g_signal_connect (priv->settings,
+                                                    "changed",
+                                                    G_CALLBACK (g_paste_settings_ui_stack_settings_changed),
+                                                    priv);
 
     GStrv *actions = priv->actions = (GStrv *) g_malloc (3 * sizeof (GStrv));
 
@@ -465,7 +466,7 @@ g_paste_settings_ui_stack_init (GPasteSettingsUiStack *self)
  *
  * Create a new instance of #GPasteSettingsUiStack
  *
- * Returns: (optional): a newly allocated #GPasteSettingsUiStack
+ * Returns: (nullable): a newly allocated #GPasteSettingsUiStack
  *                      free it with g_object_unref
  */
 G_PASTE_VISIBLE GPasteSettingsUiStack *
@@ -475,7 +476,7 @@ g_paste_settings_ui_stack_new (void)
                                                                              "margin",      12,
                                                                              "homogeneous", TRUE,
                                                                              NULL));
-    GPasteSettingsUiStackPrivate *priv = g_paste_settings_ui_stack_get_instance_private (self);
+    const GPasteSettingsUiStackPrivate *priv = _g_paste_settings_ui_stack_get_instance_private (self);
 
     if (g_paste_settings_ui_check_connection_error (priv->init_error))
     {
