@@ -1,11 +1,13 @@
 /*
  * This file is part of GPaste.
  *
- * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-ui-edit-item.h>
 
+#include "gpaste-gtk-compat.h"
+
 struct _GPasteUiEditItem
 {
     GPasteUiItemAction parent_instance;
@@ -54,7 +56,9 @@ on_item_ready (GObject      *source_object,
     gtk_scrolled_window_set_min_content_height (sw, 300);
     gtk_scrolled_window_set_min_content_width (sw, 600);
     gtk_container_add (GTK_CONTAINER (sw), text);
-    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (d)), scroll, TRUE, TRUE, 0);
+    gtk_widget_set_vexpand (scroll, TRUE);
+    gtk_widget_set_valign (scroll, TRUE);
+    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (d)), scroll, TRUE, TRUE);
     gtk_widget_show_all (scroll);
 
     if (gtk_dialog_run (d) == GTK_RESPONSE_OK)
